# Working with environmental data
# Created on April 18th, 2020
# Stefano Martellos
# martelst@units.it

# load the library we need
library(sp)
library(rgdal)
library(raster)
library(dismo)
library(maptools)

# set the working directory
setwd("~/Desktop/Lavoro/Didattica/Corso_ENM/2021/class05/exercises/GIS")

bio3 <-raster("grd/bio3")
# BIO3 = Isothermality
bio7 <-raster("grd/bio7")
# BIO7 = Temperature Annual Range
bio11 <-raster("grd/bio11")
# BIO11 = Mean Temperature of Coldest Quarter
bio12 <-raster("grd/bio12")
# BIO12 = Annual Precipitation

bbox(bio7); ncol(bio7); nrow(bio7); res(bio7)
# Wordclim data have been loaded at 1.5° lat/lon resolution, which
# roughly corresponds to a 167 km spatial resolution at the equator.

# elev <- raster("world_DEM.tif")
# bbox(elev); ncol(elev); nrow(elev); res(elev)
# here the resolution if far better, given the number of rows and cloumns.
# each pixel is far smaller than be4, and the resolution 180 times higher,
# which means that each pixel is ca 1x1 km.

# For using raster layers together in modelling, they must have the same
# resolution. Thus we can resample elev, by usign this command. 
# here the resampling is carried out using the nearest neighbor method.
# But it takes time....
# elev1<-resample(elev,bio7,method=“ngb”)

# ....so we can load a low-res one
elev1 <- raster("world_DEM_res15.tif")
bbox(elev1); ncol(elev1); nrow(elev1); res(elev1)

# now we can plot one of them
plot(bio12,main="Bio.12")

# terrain.colors() option is the default color theme in the raster package.
# Other standard themes are topo.colors(), rainbow(), and heat.colors()
# let's set the space in three rows and one column. 
par(mfrow=c(3,1))
# The par(mfrow) function is handy for creating a simple multi-paneled plot.
# mfrow is a vector of lenght 2, the first for rows and the second for columns.

# and let's plot one under the other
plot(elev1,col=rev(topo.colors(50)),main="Elevation")
plot(bio3,col=heat.colors(100),main="Bio.3")
plot(bio11,col=rainbow(100),main="Bio.11")

# Now I want two rows and two columns
par(mfrow=c(2,2))
# and let's plot correlation graphs between raster layers
plot(bio3,bio12,xlab="bio3",ylab="bio12",col="gray55")
plot(bio3,bio7,xlab="bio3",ylab="bio7",col="gray55")
plot(bio3,bio11,xlab="bio3",ylab="bio11",col="gray55")
plot(bio3,elev1,xlab="bio3",ylab="elevation",col="gray55") 

# Another example: let's see the correlation between elevation and
# annual precipitation
# let's go back to a single box layout
par(mfrow=c(1,1))
# and plot again
plot(bio12,elev1,xlab="bio12",ylab="elevation",col="gray55") 

# single box layout
par(mfrow=c(1,1))
# plot elevation raster, and contour lines at 7 levels
plot(elev1)
contour(elev1, nlevels=7, levels=c(0, 500, 1000, 1500, 2000, 3000, 4000, 5000), add=T, labels="", lwd=.2)
# The contour funcion creates isolines from the DEM
# we used the high res elev, in order to have more detaile isolines
# The contour function can also be used to generate probability isolines from
# SDM output rasters, or other isolines from any numerical or continuous 
# raster surface. It is thus a very practical command for adding lines to 
# a raster to better visualize the results.

# we can crop the raster to a given area, such as north or south America
# we are giving the coordinates of the bottom left and top right vertexes
# of a rectangle.
elev_sa<-crop(elev1, extent(-85,-30,-60,15))
elev_na<-crop(elev1, extent(-188,-50,15,90))
plot(elev_sa, main="Elevation Contours South America")
contour(elev_sa, nlevels=7, levels=c(0, 500, 1000, 1500, 2000, 3000, 4000, 5000), add=T, labels="", lwd=0.2)

# we can store the global contour line as a spatial object, and finally write
# it into a line shapefile. To do this, we use the rasterToContour() command
# in the raster package.
iso<-rasterToContour(elev1, nlevels=7, levels=c(0, 500, 1000, 1500, 2000, 3000, 4000, 5000))
# We then store the lines as shapefiles using the writeOGR() command
# from the rgdal package.
writeOGR(iso, dsn="globe", layer="isolines", driver="ESRI Shapefile", check_exists=T, overwrite_layer=T)
# This command write a vector shape file on the hard drive.
# Parameters are:
# a) the name ogf the ptial object we are storing in the file
# b) the directory to ìwhich save the shape file
# c) the name of the file
# d) the driver, i.e. one of the formats of vector shapefiles one can use
# plus some additional checks.

# it is possible to read this (or other) shapefiles using the simple 
# shapefile() command from the raster package.
iso1 <- shapefile("globe/isolines")
# now we can plot a map (north America)...
plot(elev_na,col=terrain.colors(40),main="Elevation Contours")
# ...and the shapefile containing the elevation contours
lines(iso1, lwd=0.2)


# We can express the mean temperature of the coldest quarter (bio11) from a
# sample of pixels in the climate map as a function of elevation, latitude, 
# and longitude. 
# We can then apply this function to each cell of the elevation grid and then
# plot the resulting grid. 
# To implement this calculation, we first need to load two raster files (lat,
# lon), and calculate the difference between the observed and the modeled
# mean temperature of the coldest quarter
#First, let's load the two raster lat and lon, and plot them
lat<-raster("latitude.tif")
lon<-raster("longitude.tif")
par(mfrow=c(2,1))
plot(lat)
plot(lon)

# Now, we calculate the mean temp of the coldest quartes from a sample of 
# pixels as a function of elevation, latitude and longitude
tcold<-4.138e+02 + (-3.624e-02 * elev1) + (-8.216e+00 * abs(lat)) + (-1.794e+00 * abs(lon)) + (7.122e-03 * lon^2)
# PS: don't ask me what the formula means... I took it as it was... :-D
# and then we make the difference between bio11 (measured mean temp of the 
# coldest quarter) and the calculated mean temp
diff_obs_model_temp <- bio11 - tcold
# now we can plot the results
par(mfrow=c(2,1))
plot(tcold, col=rev(rainbow(100))[20:100],main="Modelled mean temperature of the coldest quarter")
contour(elev1, nlevels=7, levels=c(0, 500, 1000, 1500, 2000, 3000, 4000, 5000), add=T, labels="", lwd=.3)
plot(diff_obs_model_temp, col=rev(rainbow(100))[20:100],main= "Difference between modelled and observed temperatures")
contour(elev1, nlevels=7, levels=c(0, 500, 1000, 1500, 2000, 3000, 4000, 5000), add=T, labels="", lwd=.3)

# we can also calculate slope and aspect (i.e., the compass direction that a
# slope faces) starting from a DEM
# from these two, we can generate an hillshade model, which can be stored and
# used for further elaboration
# here R often crashes.... I am using the hillsade tiff produced on another PC
# slope and aspect are generated using the function terrain() of the raster
# package, and the DEM elev
# slope <- terrain(elev, opt='slope')
# aspect <- terrain(elev, opt='aspect')
# the funcion hillshade() of the raster package uses slope and aspect, and two 
# further inputs: the elevation angle of the light source (sun), in degrees,
# in our case 30, and the direction (azimuth) angle of the light source (sun), 
# in degrees, in our case 315
# hillshade <- hillShade(slope, aspect, 30, 315)
# writeRaster(hillshade, "hillshade.tif", overwrite=T)

### Stack raster files
# in order to operate on several raster files at the same time (e.g. for 
# cropping purposes), we can stak them in a raster stack
# only raster files with the same resolution, and extent can be stacked
world.stk <- stack(elev1,bio3,bio7,bio11,bio12)
# we can see some statistics about the raster in the stack
summary(world.stk)
names(world.stk)
# we can plot some of them addfressing them with their id in the stack
plot(world.stk[[2:5]], col=rainbow(100,start=.0,end=.8))
# and we can change the resolution of all the raster in the stack by mean of
# a unique aggregate command. In this case we are halving the resolution of
# all the raster by using the mean of the 4 aggregated cells
world.stk3 <- aggregate(world.stk,fact=2, method="mean")
plot(world.stk3[[2:5]], col=rainbow(100,start=.0,end=.8))
# we can appreciate the grain difference from previous plot


